<?php

/**
 * Model class to give access to service configuration
 *
 * This file is part of the Kolab PHP Utilities library
 *
 * @author Thomas Bruederli <bruederli@kolabsys.com>
 *
 * Copyright (C) 2013, Kolab Systems AG <contact@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

namespace Kolab;

/**
 * Wrapper class for service configuration
 */
class Config
{
    const STRING = 0;
    const BOOL   = 1;
    const INT    = 2;
    const FLOAT  = 3;
    const ARR    = 4;

    protected static $instance;

    protected $env = '';
    protected $basedir = '../config';
    protected $data = array();
    protected $valid = false;

    /**
     * Singelton getter
     *
     * @param string Path to load config from
     */
    public static function get_instance($env = '', $dir = null)
    {
        if (!isset(self::$instance)) {
            self::$instance = new Config($env);
            if ($dir) self::$instance->basedir = $dir;
        }

        if (!self::$instance->valid) {
            self::$instance->load('config.ini');
        }

        return self::$instance;
    }

    /**
     * Default constructor
     */
    function __construct($env = '')
    {
        $this->env = $env;
        $this->load($this->basedir . '/config.ini');
    }

    /**
     * Load config from the given .ini file
     */
    private function load($file, $use_env = true)
    {
        // check for relative path
        if (!is_readable($inifile) && is_readable($this->basedir . '/' . $inifile)) {
            $inifile = $this->basedir . '/' . $inifile;
        }

        $inifile = $this->resolve_path($file, $use_env);
        if ($raw = self::parse_ini_file($inifile, true)) {
            foreach ($raw as $section => $values) {
                $sub = null;
                if (preg_match('/^(\w+)\s+"?(.+)"?$/Ui', $section, $m)) {
                    $section = $m[1];
                    $sub = trim($m[2], '"');
                }
                if (!empty($sub) && !empty($values)) {
                    $config[$section][$sub] = $values;
                }
                else if (!empty($values) && is_array($values)) {
                    $config[$section] = $values;
                }
            }

            $this->register($config);
            $this->valid = !empty($this->data);
        }
        else {
            trigger_error("Failed to parse configuration from $inifile", E_USER_ERROR);
        }
    }

    /**
     * Helper method to resolve the absolute path to the given config file.
     * This also takes the 'env' property into account.
     */
    private function resolve_path($file, $use_env)
    {
        if ($file[0] != '/' && $this->basedir[0] == '/') {
            $file = realpath($this->basedir . '/' . $file);
        }

        // check if <file>-env.ini exists
        if ($file && $use_env && !empty($this->env)) {
            $envfile = preg_replace('/\.(ini|conf)$/', '-' . $this->env . '.\\1', $file);
            if (is_file($envfile))
                return $envfile;
        }
        
        return $file;
    }

    /**
     * Replacement for PHP's parse_ini_file()
     */
    private static function parse_ini_file($filename)
    {
        $raw = array();
        foreach (file($filename) as $_line) {
            if ($_line[0] == ';')  // skip comments
                continue;

            if (preg_match('/^\[([a-z0-9-_\.]+[^\]]*)\]/', $_line, $matches)) {
                $_cur_section = $matches[1];
                $raw[$_cur_section] = array();
                unset($_cur_key);
            }

            if (preg_match('/^([a-z0-9\.-_]+)\s*=\s*(.*)/', $_line, $matches)) {
                if (isset($_cur_section) && !empty($_cur_section)) {
                    $_cur_key = $matches[1];
                    $raw[$_cur_section][$matches[1]] = isset($matches[2]) ? trim($matches[2], ' "') : '';
                }
            }
            else if (preg_match('/^\s+(.*)$/', $_line, $matches)) {
                if (isset($_cur_key) && !empty($_cur_key)) {
                    $raw[$_cur_section][$_cur_key] .=  $matches[1];
                }
            }
        }

        return $raw;
    }

    /**
     * Dump the hierarchical structure of config options into a flat list with keys delimited by dots
     */
    private function register($config, $prefix = '')
    {
        // merge the new config values over existing data
        if (empty($prefix)) {
            $this->data = array_replace_recursive($this->data, $config);
        }
        else if (is_array($config)) {
            $pkey = rtrim($prefix, '.');
            $this->data[$pkey] = is_array($this->data[$pkey]) ? array_replace_recursive((array)$this->data[$pkey], $config) : $config;
        }

        foreach ((array)$config as $key => $val) {
            if (is_array($val)) {
                $this->register($val, "$prefix$key.");
            }
            else {
                $this->data[$prefix.$key] = $val;
            }
        }

        // resolve references in config options (e.g. %(foo.bar))
        if (empty($prefix)) {
            array_walk_recursive($this->data, array($this, 'resolve_reference'));
        }
    }

    /**
     * Callback to resolve references in the given config option value
     */
    private function resolve_reference(&$value, $key)
    {
        if (is_string($value)) {
            $value = preg_replace_callback('/%[({]([\w.]+)[})]/i', array($this, 'replace_reference'), $value);
        }
    }

    /**
     * Callback function to replace the given reference with the read config value
     */
    private function replace_reference($m)
    {
        return $this->data[$m[1]];
    }

    /**
     * Magic getter for direct read-only access to config options
     */
    public function __get($name)
    {
        return $this->data[$name];
    }

    /**
     * Magic isset check
     */
    public function __isset($name)
    {
        return array_key_exists($name, $this->data);
    }

    /**
     * Common getter for config options with fallback to default values
     *
     * @param string  Config option name
     * @param mixed   Default value if option isn't set in config
     * @param integer Expected variable type
     * @return mixed Config option value
     */
    public function get($name, $default = null, $type = null)
    {
        switch ($name) {
            case 'output.tempdir':
            case 'session.savepath':
                // return an absolute path for relative directory properties
                if (isset($this->data[$name]) && $this->data[$name][0] != '/') {
                    $value = realpath(INSTALL_PATH . '/' . $this->data[$name]);
                    break;
                }

            default:
                $value = array_key_exists($name, $this->data) ? $this->data[$name] : $default;
        }

        // convert value to the requested type
        return $type ? self::convert($value, $type) : $value;
    }

    /**
     * Determines whether we have a valid configuration loaded
     *
     * @return boolean True if valid, False otherwise
     */
    public function valid()
    {
        return !empty($this->data);
    }

    /**
     * Convert the given (string) value to the requested type
     *
     * @param string  Config value
     * @param int     Output type (one of this class constants)
     * @return mixed  The converted value
     */
    public static function convert($value, $type)
    {
        // convert value to the requested type
        switch ($type) {
            case self::INT:
                return intval($value);
            case self::FLOAT:
                return floatval($value);
            case self::BOOL:
                return (bool)preg_match('/^(true|1|on|enabled|yes)$/i', $value);
            case self::ARR:
                return preg_split('/,\s*/', strval($value));
        }

        return $value;
    }
}

